<?php

/*======================================================================*\
|| #################################################################### ||
|| # JAKCMS PRO 2.2.5                                                 # ||
|| # ---------------------------------------------------------------- # ||
|| # Copyright 2011 JAKCMS All Rights Reserved.                       # ||
|| # This file may not be redistributed in whole or significant part. # ||
|| # ---------------- JAKCMS IS NOT FREE SOFTWARE ----------------    # ||
|| # http://www.jakcms.com | http://www.jakcms.com/license-agreement  # ||
|| #################################################################### ||
\*======================================================================*/

/**
 *  RainTPL
 *  -------
 *	Realized by Federico Ulfo & maintained by the Rain Team
 *	Distributed under GNU/LGPL 3 License
 *
 *  @version 2.6.4
 */

class JAKCache{

		static $tpl_dir = "template/";
		static $cache_dir = "cache/";
		static $tpl_ext = "html";
		static $black_list = array( '\$this', 'jakcache::', 'self::', '_SESSION', '_SERVER', '_ENV',  'eval', 'exec', 'unlink', 'rmdir' );
		static $check_template_update = true;

		// Is the array where JAKCache keep the variables assigned
		public $var = array();
		private $tpl = array(),		// variables to keep the template directories and info
				$cache = false,		// static cache enabled / disabled
                $cache_id = null;   // identify only one cache
        private static $config_name_sum = null;

	// -------------------------

		const CACHE_EXPIRE_TIME = 3600; // default cache expire time = hour



	// Assign variable
	function assign($variable, $value = null) {
		if( is_array( $variable ) )
			$this->var += $variable;
		else
			$this->var[ $variable ] = $value;
	}

	/**
	 * Draw the template
	 * eg. 	$html = $tpl->draw( 'demo', TRUE ); // return template in string
	 * or 	$tpl->draw( $tpl_name ); // echo the template
	 *
	 * @param string $tpl_name  template to load
	 * @param boolean $return_string  true=return a string, false=echo the template
	 * @return string
	 */

	function draw($tpl_name, $return_string = false) {

		// compile the template if necessary and set the template filepath
		$this->check_template($tpl_name);


		// Cache is off and, return_string is false
                // Rain just echo the template

                if (!$this->cache && !$return_string) {
                        extract($this->var);
            		include $this->tpl['compiled_filename'];
                        unset($this->tpl);
                }


		// cache or return_string are enabled
                // rain get the output buffer to save the output in the cache or to return it as string

                else {

                        //----------------------
                        // get the output buffer
                        //----------------------
			ob_start();
			extract($this->var);
			include $this->tpl['compiled_filename'];
			$jakcache_contents = ob_get_contents();
			ob_end_clean();
                        //----------------------


                        // save the output in the cache
			if ($this->cache)
				file_put_contents($this->tpl['cache_filename'], "<?php if(!class_exists('jakcache')){exit;}?>" . $jakcache_contents);

                        // free memory
                        unset($this->tpl);

                        // return or print the template
                        if ($return_string) return $jakcache_contents; else echo $jakcache_contents;

                }

	}

	/**
	 * If exists a valid cache for this template it returns the cache
	 *
	 * @param string $tpl_name Name of template (set the same of draw)
	 * @param int $expiration_time Set after how many seconds the cache expire and must be regenerated
	 * @return string it return the HTML or null if the cache must be recreated
	 */

	function cache($tpl_name, $expire_time = self::CACHE_EXPIRE_TIME, $cache_id = null) {

                // set the cache_id
                $this->cache_id = $cache_id;

		if (!$this->check_template( $tpl_name ) && file_exists($this->tpl['cache_filename']) && ( time() - filemtime($this->tpl['cache_filename']) < $expire_time))
			return substr(file_get_contents( $this->tpl['cache_filename']), 43);
		else {
			//delete the cache of the selected template
                        if (file_exists($this->tpl['cache_filename']))
                            unlink($this->tpl['cache_filename'] );
			$this->cache = true;
		}
	}

	/**
	 * Configure the settings of JAKCache
	 *
	 */
	static function configure( $setting, $value = null ){
		if( is_array( $setting ) )
			foreach( $setting as $key => $value )
				self::configure( $key, $value );
		else if( property_exists( __CLASS__, $setting ) ){
			self::$$setting = $value;
	                    self::$config_name_sum .= $value; // take trace of all config
	            }
	}

	// check if has to compile the template
	// return true if the template has changed
	private function check_template( $tpl_name ){

		if (!isset($this->tpl['checked'])) {

			$tpl_basename                       = basename( $tpl_name );														// template basename
			$tpl_basedir                        = strpos($tpl_name,"/") ? dirname($tpl_name) . '/' : null;						// template basedirectory
			$tpl_dir                            = self::$tpl_dir . $tpl_basedir;								// template directory
			$this->tpl['tpl_filename']          = $tpl_dir . $tpl_basename . '.' . self::$tpl_ext;	// template filename
			$temp_compiled_filename             = self::$cache_dir . $tpl_basename . "." . md5( $tpl_dir . self::$config_name_sum );
			$this->tpl['compiled_filename']     = $temp_compiled_filename . '.php';	// cache filename
			$this->tpl['cache_filename']        = $temp_compiled_filename . '.s_' . $this->cache_id . '.php';	// static cache filename

			// if the template doesn't exsist throw an error
			if (self::$check_template_update && !file_exists($this->tpl['tpl_filename'])) {
				trigger_error('Template <b>'.$this->tpl['tpl_filename'].'</b> not found!' );
				return '<div style="background:#f8f8ff;border:1px solid #aaaaff;padding:10px;">Template <b>'.$this->tpl['tpl_filename'].'</b> not found</div>';
			}

			// file doesn't exsist, or the template was updated, Rain will compile the template
			if( !file_exists($this->tpl['compiled_filename']) || ( self::$check_template_update && filemtime($this->tpl['compiled_filename']) < filemtime( $this->tpl['tpl_filename']))) {
				$this->compileFile( $tpl_basename, $tpl_basedir, $this->tpl['tpl_filename'], self::$cache_dir, $this->tpl['compiled_filename']);
				return true;
			}
			$this->tpl['checked'] = true;
		}
	}


	/**
	* execute stripslaches() on the xml block. Invoqued by preg_replace_callback function below
	* @access private
	*/
	private function xml_reSubstitution($capture) {
    		return "<?php echo '<?xml ".stripslashes($capture[1])." ?>'; ?>";
	} 

	/**
	 * Compile and write the compiled template file
	 * @access private
	 */
	private function compileFile($tpl_basename, $tpl_basedir, $tpl_filename, $cache_dir, $compiled_filename) {

		//read template file
		$this->tpl['source'] = $template_code = file_get_contents($tpl_filename);

		//xml substitution
		$template_code = preg_replace("/<\?xml(.*?)\?>/s", "##XML\\1XML##", $template_code);

		//xml re-substitution
		$template_code = preg_replace_callback ("/##XML(.*?)XML##/s", array($this, 'xml_reSubstitution'), $template_code); 

		//compile template
		$template_compiled = "<?php if(!class_exists('jakcache')){exit;}?>" . $this->compileTemplate($template_code, $tpl_basedir);
		

		// fix the php-eating-newline-after-closing-tag-problem
		$template_compiled = str_replace("?>\n", "?>\n\n", $template_compiled);

		// create directories
		if( !is_dir($cache_dir))
			mkdir($cache_dir, 0755, true);

		if( !is_writable( $cache_dir ) )
			die("Cache directory <b>$cache_dir</b> doesn't have write permission. Set write permission. More details on <a target=_blank href=http://www.jakcms.com>JAKCMS</a>");

		//write compiled file
		file_put_contents( $compiled_filename, $template_compiled );
	}

	/**
	 * Compile template
	 * @access private
	 */
	private function compileTemplate( $template_code, $tpl_basedir ){

		//tag list
		$tag_regexp = array('include' => '(\{include="[^"]*"(?: cache="[^"]*")?\})',);

		$tag_regexp = "/" . join("|", $tag_regexp) . "/";

		//split the code with the tags regexp
		$template_code = preg_split ($tag_regexp, $template_code, -1, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);

		//compile the code
		$compiled_code = $this->compileCode($template_code);

		//return the compiled code
		return $compiled_code;

	}

	/**
	 * Compile the code
	 * @access private
	 */
	private function compileCode($parsed_code) {

		//variables initialization
		$compiled_code = $open_if = $comment_is_open = $ignore_is_open = null;
                $loop_level = 0;


	 	//read all parsed code
	 	while( $html = array_shift( $parsed_code ) ){

	 		//include files
			if( preg_match('/(?:\{include="([^"]*)"(?: cache="([^"]*)"){0,1}\})/', $html, $code)) {

				//variables substitution
				$include_var = $this->var_replace($code[1], $left_delimiter = '\{', $right_delimiter = '\}', $php_left_delimiter = '".' , $php_right_delimiter = '."', $loop_level);

				// if the cache is active
				if( isset($code[ 2 ]) )
					//dynamic include
					$compiled_code .= '<?php $tpl = new JAKCache;' .
								 'if( $cache = $tpl->cache($template = basename("'.$include_var.'")))' .
								 '	echo $cache;' .
								 'else{ ' .
								 '$tpl_dir_temp = self::$tpl_dir;' .
								 '$tpl->assign($this->var);' .
								 'self::$tpl_dir .= dirname("'.$include_var.'") . (substr("'.$include_var.'",-1,1) != "/" ? "/" : "");' .
								 (!$loop_level ? null : '$tpl->assign( "key", $key'.$loop_level.' ); $tpl->assign("value", $value'.$loop_level.');').
								 '$tpl->draw($template);'.
								 'self::$tpl_dir = $tpl_dir_temp;' .
								 '}' .
								 '?>';
				else
					//dynamic include
					$compiled_code .= '<?php $tpl = new JAKCache;' .
								 '$tpl_dir_temp = self::$tpl_dir;' .
								 '$tpl->assign($this->var);' .
								 'self::$tpl_dir .= dirname("'.$include_var.'") . ( substr("'.$include_var.'",-1,1) != "/" ? "/" : "" );' .
								 (!$loop_level ? null : '$tpl->assign("key", $key'.$loop_level.'); $tpl->assign("value", $value'.$loop_level.');').
								 '$tpl->draw(basename("'.$include_var.'"));'.
								 'self::$tpl_dir = $tpl_dir_temp;' .
								 '?>';

			}

			//all html code
			else{

				//variables substitution (es. {$title})
				$html = $this->var_replace( $html, $left_delimiter = '\{', $right_delimiter = '\}', $php_left_delimiter = '<?php ', $php_right_delimiter = ';?>', $loop_level, $echo = true );
				//functions substitution (es. {"string"|functions})
				$compiled_code .= $this->func_replace( $html, $left_delimiter = '\{', $right_delimiter = '\}', $php_left_delimiter = '<?php ', $php_right_delimiter = ';?>', $loop_level, $echo = true );
			}
		}

		if( $open_if > 0 )
			die("Error! You need to close an {if} tag in <b>". $this->tpl['tpl_filename'] ." </b>template");

		return $compiled_code;
	}

	/**
	 * replace the path of image src, link href and a href.
	 * url => template_dir/url
	 * url# => url
	 * http://url => http://url
	 *
	 * @param string $html
	 * @return string html sostituito
	 */
	
	// replace const
	function const_replace( $html, $tag_left_delimiter, $tag_right_delimiter, $php_left_delimiter = null, $php_right_delimiter = null, $loop_level = null, $echo = null ){
		// const
		return preg_replace('/\{\#(\w+)\#{0,1}\}/', $php_left_delimiter.($echo ? " echo " : null) . '\\1' . $php_right_delimiter, $html);
	}

	// replace functions/modifiers on constants and strings
	function func_replace( $html, $tag_left_delimiter, $tag_right_delimiter, $php_left_delimiter = null, $php_right_delimiter = null, $loop_level = null, $echo = null ){

		preg_match_all('/' . '\{\#{0,1}(\"{0,1}.*?\"{0,1})(\|\w.*?)\#{0,1}\}' . '/', $html, $matches);

		for( $i=0, $n=count($matches[0]); $i<$n; $i++ ){

			//complete tag ex: {$news.title|substr:0,100}
			$tag = $matches[ 0 ][ $i ];

			//variable name ex: news.title
			$var = $matches[ 1 ][ $i ];

			//function and parameters associate to the variable ex: substr:0,100
			$extra_var = $matches[ 2 ][ $i ];

			// check if there's any function disabled by black_list
			$this->function_check($tag);

			$extra_var = $this->var_replace($extra_var, null, null, null, null, $loop_level);

			// check if there's an operator = in the variable tags, if there's this is an initialization so it will not output any value
			$is_init_variable = preg_match("/^(\s*?)\=[^=](.*?)$/", $extra_var);

			//function associate to variable
			$function_var = ($extra_var and $extra_var[0] == '|') ? substr($extra_var, 1) : null;

			//variable path split array (ex. $news.title o $news[title]) or object (ex. $news->title)
			$temp = preg_split("/\.|\[|\-\>/", $var);

			//variable name
			$var_name = $temp[ 0 ];

			//variable path
			$variable_path = substr($var, strlen( $var_name));

			//parentesis transform [ e ] in [" e in "]
			$variable_path = str_replace('[', '["', $variable_path);
			$variable_path = str_replace(']', '"]', $variable_path);

			//transform .$variable in ["$variable"]
			$variable_path = preg_replace('/\.\$(\w+)/', '["$\\1"]', $variable_path);

			//transform [variable] in ["variable"]
			$variable_path = preg_replace('/\.(\w+)/', '["\\1"]', $variable_path);

			//if there's a function
			if( $function_var ){


                                // check if there's a function or a static method and separate, function by parameters
				$function_var = str_replace("::", "@double_dot@", $function_var);

                                // get the position of the first :
                                if( $dot_position = strpos($function_var, ":")){

                                    // get the function and the parameters
                                    $function = substr($function_var, 0, $dot_position);
                                    $params = substr($function_var, $dot_position+1);

                                }
                                else{

                                    //get the function
                                    $function = str_replace("@double_dot@", "::", $function_var);
                                    $params = null;

                                }

                                // replace back the @double_dot@ with ::
                                $function = str_replace("@double_dot@", "::", $function);
                                $params = str_replace("@double_dot@", "::", $params);


			}
			else
				$function = $params = null;

			$php_var = $var_name . $variable_path;

			// compile the variable for php
			if (isset($function )) {
				if( $php_var )
					$php_var = $php_left_delimiter . (!$is_init_variable && $echo ? 'echo ' : null) . ($params ? "($function($php_var, $params))" : "$function($php_var)" ) . $php_right_delimiter;
				else
					$php_var = $php_left_delimiter . (!$is_init_variable && $echo ? 'echo ' : null) . ($params ? "($function($params))" : "$function()") . $php_right_delimiter;
			}
			else
				$php_var = $php_left_delimiter . (!$is_init_variable && $echo ? 'echo ' : null) . $php_var . $extra_var . $php_right_delimiter;

			$html = str_replace($tag, $php_var, $html);

		}

		return $html;

	}

	function var_replace( $html, $tag_left_delimiter, $tag_right_delimiter, $php_left_delimiter = null, $php_right_delimiter = null, $loop_level = null, $echo = null ){

		//all variables
		if( preg_match_all( '/' . $tag_left_delimiter . '\$(\w+(?:\.\${0,1}[A-Za-z0-9_]+)*(?:(?:\[\${0,1}[A-Za-z0-9_]+\])|(?:\-\>\${0,1}[A-Za-z0-9_]+))*)(.*?)' . $tag_right_delimiter . '/', $html, $matches ) ){

                    for( $parsed=array(), $i=0, $n=count($matches[0]); $i<$n; $i++ )
                        $parsed[$matches[0][$i]] = array('var'=>$matches[1][$i],'extra_var'=>$matches[2][$i]);

                    foreach( $parsed as $tag => $array ){

                            //variable name ex: news.title
                            $var = $array['var'];

                            //function and parameters associate to the variable ex: substr:0,100
                            $extra_var = $array['extra_var'];

                            // check if there's any function disabled by black_list
                            $this->function_check( $tag );

                            $extra_var = $this->var_replace( $extra_var, null, null, null, null, $loop_level );

                            // check if there's an operator = in the variable tags, if there's this is an initialization so it will not output any value
                            $is_init_variable = preg_match( "/^(\s*?)\=[^=](.*?)$/", $extra_var );

                            //function associate to variable
                            $function_var = ( $extra_var and $extra_var[0] == '|') ? substr( $extra_var, 1 ) : null;

                            //variable path split array (ex. $news.title o $news[title]) or object (ex. $news->title)
                            $temp = preg_split( "/\.|\[|\-\>/", $var );

                            //variable name
                            $var_name = $temp[ 0 ];

                            //variable path
                            $variable_path = substr( $var, strlen( $var_name ) );

                            //parentesis transform [ e ] in [" e in "]
                            $variable_path = str_replace( '[', '["', $variable_path );
                            $variable_path = str_replace( ']', '"]', $variable_path );

                            //transform .$variable in ["$variable"]
                            $variable_path = preg_replace('/\.\$(\w+)/', '["$\\1"]', $variable_path );

                            //transform [variable] in ["variable"]
                            $variable_path = preg_replace('/\.(\w+)/', '["\\1"]', $variable_path );

                            //if there's a function
                            if( $function_var ){

                                    //split function by function_name and parameters (ex substr:0,100)
                                    $function_split = explode( ':', $function_var, 2 );

                                    //function name
                                    $function = $function_split[ 0 ];

                                    //function parameters
                                    $params = ( isset( $function_split[ 1 ] ) ) ? $function_split[ 1 ] : null;

                            }
                            else
                                    $function = $params = null;

                            //if it is inside a loop
                            if( $loop_level ){
                                    //verify the variable name
                                    if( $var_name == 'key' )
                                            $php_var = '$key' . $loop_level;
                                    elseif( $var_name == 'value' )
                                            $php_var = '$value' . $loop_level . $variable_path;
                                    elseif( $var_name == 'counter' )
                                            $php_var = '$counter' . $loop_level;
                                    else
                                            $php_var = "\$" . $var_name . $variable_path;
                            }else
                                    $php_var = "\$" . $var_name . $variable_path;

                            // compile the variable for php
                            if( isset( $function ) )
                                    $php_var = $php_left_delimiter . (!$is_init_variable && $echo ? 'echo ' : null) . ($params ? "($function($php_var, $params))" : "$function($php_var)") . $php_right_delimiter;
                            else
                                    $php_var = $php_left_delimiter . (!$is_init_variable && $echo ? 'echo ' : null) . $php_var . $extra_var . $php_right_delimiter;

                            $html = str_replace($tag, $php_var, $html);


                    }
                }

		return $html;
	}

	/**
	 * Check if function is in black list (sandbox)
	 *
	 * @param string $code
	 * @param string $tag
	 */
	private function function_check( $code ){

		$preg = '#(\W|\s)' . implode( '(\W|\s)|(\W|\s)', self::$black_list ) . '(\W|\s)#';

		// check if the function is in the black list (or not in white list)
		if( count(self::$black_list) && preg_match( $preg, $code, $match ) ){

			// find the line of the error
			$line = 0;
			$rows=explode("\n",$this->tpl['source']);
			while( !strpos($rows[$line],$code) )
				$line++;

			// draw the error line
			$error = str_replace( array('<','>'), array( '&lt;','&gt;' ), array($code,$rows[$line]) );
			$error = str_replace( $code, "<font color=red>$code</font>", $rows[$line] );

			// debug the error and stop the execution of the script
			die( "<div>JAKCache Sandbox Error in template <b>{$this->tpl['tpl_filename']}</b> at line $line : <i>$error</i></b>" );
		}

	}

}
?>